@@ -23,7 +23,11 @@
 #include <gr_cpu.h>
 
 #include <gr_fir_ccf.h>
+#include <gr_fir_ccf_generic.h>
+#include <gr_fir_ccf_x86.h>
 #include <gr_fir_fcc.h>
+#include <gr_fir_fcc_generic.h>
+#include <gr_fir_fcc_x86.h>
 #include <gr_fir_fff.h>
 #include <gr_fir_fff_generic.h>
 #include <gr_fir_fff_x86.h>
@@ -51,6 +55,30 @@ using std::cerr;
  * ----------------------------------------------------------------
  */
 
+static gr_fir_ccf *
+make_gr_fir_ccf_3dnow(const std::vector<float> &taps)
+{
+  return new gr_fir_ccf_3dnow(taps);
+}
+
+static gr_fir_ccf *
+make_gr_fir_ccf_sse(const std::vector<float> &taps)
+{
+  return new gr_fir_ccf_sse(taps);
+}
+
+static gr_fir_fcc *
+make_gr_fir_fcc_3dnow(const std::vector<gr_complex> &taps)
+{
+  return new gr_fir_fcc_3dnow(taps);
+}
+
+static gr_fir_fcc *
+make_gr_fir_fcc_sse(const std::vector<gr_complex> &taps)
+{
+  return new gr_fir_fcc_sse(taps);
+}
+
 static gr_fir_ccc *
 make_gr_fir_ccc_3dnow (const std::vector<gr_complex> &taps)
 {
@@ -141,6 +169,62 @@ make_gr_fir_scc_sse(const std::vector<gr_complex> &taps)
  * ----------------------------------------------------------------
  */
 
+gr_fir_ccf *
+gr_fir_sysconfig_x86::create_gr_fir_ccf (const std::vector<float> &taps)
+{
+  static bool first = true;
+
+  if (gr_cpu::has_3dnow ()){
+    if (first){
+      cerr << ">>> gr_fir_ccf: using 3DNow!\n";
+      first = false;
+    }
+    return make_gr_fir_ccf_3dnow (taps);
+  }
+
+  if (gr_cpu::has_sse ()){
+    if (first){
+      cerr << ">>> gr_fir_ccf: using SSE\n";
+      first = false;
+    }
+    return make_gr_fir_ccf_sse (taps);
+  }
+
+  if (first){
+    cerr << ">>> gr_fir_ccf: handing off to parent class\n";
+    first = false;
+  }
+  return gr_fir_sysconfig_generic::create_gr_fir_ccf (taps);
+}
+
+gr_fir_fcc *
+gr_fir_sysconfig_x86::create_gr_fir_fcc (const std::vector<gr_complex> &taps)
+{
+  static bool first = true;
+
+  if (gr_cpu::has_3dnow ()){
+    if (first){
+      cerr << ">>> gr_fir_fcc: using 3DNow!\n";
+      first = false;
+    }
+    return make_gr_fir_fcc_3dnow (taps);
+  }
+
+  if (gr_cpu::has_sse ()){
+    if (first){
+      cerr << ">>> gr_fir_fcc: using SSE\n";
+      first = false;
+    }
+    return make_gr_fir_fcc_sse (taps);
+  }
+
+  if (first){
+    cerr << ">>> gr_fir_fcc: handing off to parent class\n";
+    first = false;
+  }
+  return gr_fir_sysconfig_generic::create_gr_fir_fcc (taps);
+}
+
 gr_fir_ccc *
 gr_fir_sysconfig_x86::create_gr_fir_ccc (const std::vector<gr_complex> &taps)
 {
@@ -300,19 +384,45 @@ gr_fir_sysconfig_x86::create_gr_fir_scc (const std::vector<gr_complex> &taps)
 void 
 gr_fir_sysconfig_x86::get_gr_fir_ccf_info (std::vector<gr_fir_ccf_info> *info)
 {
+  gr_fir_ccf_info	t;
+
   // invoke parent..
   gr_fir_sysconfig_generic::get_gr_fir_ccf_info (info);
 
-  // ...add our stuff
+  // add our stuff...
+  if (gr_cpu::has_3dnow ()){
+    t.name = "3DNow!";
+    t.create = make_gr_fir_ccf_3dnow;
+    (*info).push_back (t);
+  }
+
+  if (gr_cpu::has_sse ()){
+    t.name = "SSE";
+    t.create = make_gr_fir_ccf_sse;
+    (*info).push_back (t);
+  }
 }
 
 void 
 gr_fir_sysconfig_x86::get_gr_fir_fcc_info (std::vector<gr_fir_fcc_info> *info)
 {
+  gr_fir_fcc_info	t;
+
   // invoke parent..
   gr_fir_sysconfig_generic::get_gr_fir_fcc_info (info);
 
   // add our stuff...
+  if (gr_cpu::has_3dnow ()){
+    t.name = "3DNow!";
+    t.create = make_gr_fir_fcc_3dnow;
+    (*info).push_back (t);
+  }
+
+  if (gr_cpu::has_sse ()){
+    t.name = "SSE";
+    t.create = make_gr_fir_fcc_sse;
+    (*info).push_back (t);
+  }
 }
 
 void 
