@@ -26,8 +26,8 @@
 
 class gr_fir_sysconfig_x86 : public gr_fir_sysconfig_generic {
 public:
-//virtual gr_fir_ccf *create_gr_fir_ccf (const std::vector<float> &taps);
-//virtual gr_fir_fcc *create_gr_fir_fcc (const std::vector<gr_complex> &taps);
+  virtual gr_fir_ccf *create_gr_fir_ccf (const std::vector<float> &taps);
+  virtual gr_fir_fcc *create_gr_fir_fcc (const std::vector<gr_complex> &taps);
   virtual gr_fir_fff *create_gr_fir_fff (const std::vector<float> &taps);
   virtual gr_fir_fsf *create_gr_fir_fsf (const std::vector<float> &taps);
   virtual gr_fir_scc *create_gr_fir_scc (const std::vector<gr_complex> &taps);
