@@ -41,8 +41,7 @@ gr_hilbert_fc::gr_hilbert_fc (unsigned int ntaps)
 		   gr_make_io_signature (1, 1, sizeof (float)),
 		   gr_make_io_signature (1, 1, sizeof (gr_complex))),
     d_ntaps (ntaps | 0x1),	// ensure ntaps is odd
-    d_hilb (gr_fir_util::create_gr_fir_fff (
-	      gr_firdes::reverse (gr_firdes::hilbert (d_ntaps))))
+    d_hilb (gr_fir_util::create_gr_fir_fff (gr_firdes::hilbert (d_ntaps)))
 {
   set_history (d_ntaps);
 }
