@@ -20,7 +20,7 @@
 # Boston, MA 02111-1307, USA.
 # 
 
-from gnuradio.build_utils import output_glue
+from build_utils import output_glue
 
 import generate_common
 
