@@ -120,7 +120,7 @@ gr_check_lfsr_32k_s::enter_SEARCHING ()
   d_index = 0;			// reset LFSR to beginning
 
   if (0)
-    fprintf (stdout, "gr_check_lfsr_32k: enter_SEARCHING at offset %8ld (0x%08x)\n",
+    fprintf (stdout, "gr_check_lfsr_32k: enter_SEARCHING at offset %8ld (0x%08lx)\n",
 	     d_ntotal, d_ntotal);
 
   enter_MATCH0 ();
@@ -155,7 +155,7 @@ gr_check_lfsr_32k_s::enter_LOCKED ()
   d_index = 3;			// already matched first 3 items
 
   if (0)
-    fprintf (stdout, "gr_check_lfsr_32k: enter_LOCKED at offset %8ld (0x%08x)\n",
+    fprintf (stdout, "gr_check_lfsr_32k: enter_LOCKED at offset %8ld (0x%08lx)\n",
 	     d_ntotal, d_ntotal);
 }
 
@@ -164,6 +164,6 @@ gr_check_lfsr_32k_s::log_error (unsigned short expected, unsigned short actual)
 {
   if (0)
     fprintf (stdout, 
-     "gr_check_lfsr_32k: expected %5d (0x%04x) got %5d (0x%04x) offset %8d (0x%08x)\n",
+     "gr_check_lfsr_32k: expected %5d (0x%04x) got %5d (0x%04x) offset %8ld (0x%08lx)\n",
 	     expected, expected, actual, actual, d_ntotal, d_ntotal);
 }
