@@ -25,6 +25,7 @@
 
 #include <vector>
 #include <cmath>
+#include <gr_complex.h>
 
 /*!
  * \brief Finite Impulse Response (FIR) filter design functions.
@@ -37,7 +38,8 @@ class gr_firdes {
     WIN_HAMMING = 0,	// max attenuation 53 dB
     WIN_HANN = 1,	// max attenuation 44 dB
     WIN_BLACKMAN = 2,	// max attenuation 74 dB
-    WIN_RECTANGULAR = 3
+    WIN_RECTANGULAR = 3,
+    WIN_KAISER = 4      // max attenuation a function of beta, google it
   };
 
   // ... class methods ...
@@ -166,24 +168,20 @@ class gr_firdes {
    * \brief design a Gaussian filter
    *
    * \p gain:			overall gain of filter (typically 1.0)
-   * \p sampling_freq:		sampling freq (Hz)
-   * \p symbol rate:		symbol rate, must be a factor of sample rate
+   * \p symbols per bit:	symbol rate, must be a factor of sample rate
    * \p ntaps:		        number of taps
    */
   static std::vector<float>
   gaussian (double gain,
-	    double sampling_freq,
-	    double symbol_rate,       
+	    double spb,       
 	    double bt,              // Bandwidth to bitrate ratio
 	    int ntaps);
 
   // window functions ...
   static std::vector<float> gr_firdes::window (win_type type, int ntaps, double beta);
 
-  // reverse the order of taps
-  static std::vector<float> reverse (const std::vector<float> &taps);
-
 private:
+  static double bessi0(double x);
   static void sanity_check_1f (double sampling_freq, double f1,
 			       double transition_width);
   static void sanity_check_2f (double sampling_freq, double f1, double f2,
