@@ -30,6 +30,10 @@ const float gr_fxpt::s_sine_table[1 << NBITS][2] = {
 #include "sine_table.h"
 };
 
+// gcc 4.x fix
+const float gr_fxpt::TWO_TO_THE_31;
+const float gr_fxpt::PI;
+
 #if 0
 /*
  * Compute sine using table lookup with linear interpolation.
