@@ -45,7 +45,9 @@ gr_block::gr_block (const std::string &name,
     d_output_signature (output_signature),
     d_output_multiple (1),
     d_relative_rate (1.0),
-    d_unique_id (s_next_id++)
+    d_unique_id (s_next_id++),
+    d_history(1),
+    d_fixed_rate(false)
 {
   s_ncurrently_allocated++;
 }
@@ -65,7 +67,6 @@ gr_block::forecast (int noutput_items, gr_vector_int &ninput_items_required)
     ninput_items_required[i] = noutput_items;
 }
 
-
 // default implementation
 
 bool
@@ -74,6 +75,18 @@ gr_block::check_topology (int ninputs, int noutputs)
   return true;
 }
 
+bool
+gr_block::start()
+{
+  return true;
+}
+
+bool
+gr_block::stop()
+{
+  return true;
+}
+
 void
 gr_block::set_output_multiple (int multiple)
 {
@@ -104,3 +117,15 @@ gr_block::consume_each (int how_many_items)
 {
   d_detail->consume_each (how_many_items);
 }
+
+int
+gr_block::fixed_rate_ninput_to_noutput(int ninput)
+{
+  throw std::runtime_error("Unimplemented");
+}
+
+int
+gr_block::fixed_rate_noutput_to_ninput(int noutput)
+{
+  throw std::runtime_error("Unimplemented");
+}
