@@ -123,11 +123,11 @@ gr_buffer::allocate_buffer (int nitems, size_t sizeof_item)
   // This only happens if sizeof_item is not a power of two.
 
   if (nitems > 2 * orig_nitems && nitems * (int) sizeof_item > granularity){
-    std::cerr << "gr_buffer::allocate_buffer: warning: tried to allocate "
-	      << orig_nitems << " items of size "
-	      << sizeof_item << ".\n    Due to alignment requirements "
-	      << nitems << "were allocated.  If this isn't\n"
-	      << "   OK, consider padding your structure to a power-of-two bytes.\n"
+    std::cerr << "gr_buffer::allocate_buffer: warning: tried to allocate\n"
+	      << "   " << orig_nitems << " items of size "
+	      << sizeof_item << ". Due to alignment requirements\n"
+	      << "   " << nitems << " were allocated.  If this isn't OK, consider padding\n"
+	      << "   your structure to a power-of-two bytes.\n"
 	      << "   On this platform, our allocation granularity is " << granularity << " bytes.\n";
   }
 
@@ -178,9 +178,11 @@ gr_buffer::update_write_pointer (int nitems)
 }
 
 gr_buffer_reader_sptr
-gr_buffer_add_reader (gr_buffer_sptr buf)
+gr_buffer_add_reader (gr_buffer_sptr buf, int history)
 {
-  gr_buffer_reader_sptr r (new gr_buffer_reader (buf, buf->d_write_index));
+  gr_buffer_reader_sptr r (new gr_buffer_reader (buf,
+						 buf->index_sub(buf->d_write_index,
+								history-1)));
   buf->d_readers.push_back (r.get ());
 
   return r;
