@@ -31,17 +31,25 @@
 #include <iostream>
 #include <limits>
 #include <assert.h>
+#include <stdio.h>
 
+// must be defined to either 0 or 1
+#define ENABLE_LOGGING 0
 
-gr_single_threaded_scheduler::gr_single_threaded_scheduler (
-    const std::vector<gr_block_sptr> &blocks)
-  : d_blocks (blocks), d_enabled (true)
-{
-#if 0
-  std::cout << "gr_single_threaded_scheduler: "
-	    << d_blocks.size ()
-	    << " blocks\n";
+#if (ENABLE_LOGGING)
+#define LOG(x) do { x; } while(0)
+#else
+#define LOG(x) do {;} while(0)
 #endif
+
+static int which_scheduler  = 0;
+
+
+std::ostream&
+operator << (std::ostream& os, const gr_block *m)
+{
+  os << "<gr_block " << m->name() << " (" << m->unique_id() << ")>";
+  return os;
 }
 
 gr_single_threaded_scheduler_sptr
@@ -51,8 +59,24 @@ gr_make_single_threaded_scheduler (const std::vector<gr_block_sptr> &blocks)
     gr_single_threaded_scheduler_sptr (new gr_single_threaded_scheduler (blocks));
 }
 
+gr_single_threaded_scheduler::gr_single_threaded_scheduler (
+    const std::vector<gr_block_sptr> &blocks)
+  : d_blocks (blocks), d_enabled (true), d_log(0)
+{
+  if (ENABLE_LOGGING){
+    char name[100];
+    snprintf(name, sizeof(name), "sst-%d.log", which_scheduler++);
+    d_log = new std::ofstream(name);
+    *d_log << "gr_single_threaded_scheduler: "
+	   << d_blocks.size ()
+	   << " blocks\n";
+  }
+}
+
 gr_single_threaded_scheduler::~gr_single_threaded_scheduler ()
 {
+  if (ENABLE_LOGGING)
+    delete d_log;
 }
 
 void
@@ -111,24 +135,41 @@ gr_single_threaded_scheduler::main_loop ()
   unsigned int			bi;
   unsigned int			nalive;
   int				max_items_avail;
+  bool				made_progress_last_pass;
+  bool				making_progress;
 
   for (unsigned i = 0; i < d_blocks.size (); i++)
     d_blocks[i]->detail()->set_done (false);		// reset any done flags
 
-  nalive = d_blocks.size ();
+  for (unsigned i = 0; i < d_blocks.size (); i++)	// enable any drivers, etc.
+    d_blocks[i]->start();
+
+
+  bi = 0;
+  made_progress_last_pass = true;
+  making_progress = false;
 
   // Loop while there are still blocks alive
-  
-  bi = 0;
+
+  nalive = d_blocks.size ();
   while (d_enabled && nalive > 0){
 
     gr_block		*m = d_blocks[bi].get ();
     gr_block_detail	*d = m->detail().get ();
 
+    LOG(*d_log << std::endl << m);
+
     if (d->done ())
       goto next_block;
 
     if (d->source_p ()){
+      // Invoke sources as a last resort.  As long as the previous pass
+      // made progress, don't call a source.
+      if (made_progress_last_pass){
+	LOG(*d_log << "  Skipping source\n");
+	goto next_block;
+      }
+
       ninput_items_required.resize (0);
       ninput_items.resize (0);
       input_items.resize (0);
@@ -136,11 +177,14 @@ gr_single_threaded_scheduler::main_loop ()
 
       // determine the minimum available output space
       noutput_items = min_available_space (d, m->output_multiple ());
+      LOG(*d_log << " source\n  noutput_items = " << noutput_items << std::endl);
       if (noutput_items == -1)		// we're done
 	goto were_done;
 
-      if (noutput_items == 0)		// we're output blocked
+      if (noutput_items == 0){		// we're output blocked
+	LOG(*d_log << "  BLKD_OUT\n");
 	goto next_block;
+      }
 
       goto setup_call_to_work;		// jump to common code
     }
@@ -150,11 +194,13 @@ gr_single_threaded_scheduler::main_loop ()
       ninput_items.resize (d->ninputs ());
       input_items.resize (d->ninputs ());
       output_items.resize (0);
+      LOG(*d_log << " sink\n");
 
       max_items_avail = 0;
       for (int i = 0; i < d->ninputs (); i++){
 	ninput_items[i] = d->input(i)->items_available();
-	if (ninput_items[i] == 0 && d->input(i)->done())
+	//if (ninput_items[i] == 0 && d->input(i)->done())
+	if (ninput_items[i] < m->output_multiple() && d->input(i)->done())
 	  goto were_done;
 	
 	max_items_avail = std::max (max_items_avail, ninput_items[i]);
@@ -162,10 +208,14 @@ gr_single_threaded_scheduler::main_loop ()
 
       // take a swag at how much output we can sink
       noutput_items = (int) (max_items_avail * m->relative_rate ());
-      noutput_items = round_up (noutput_items, m->output_multiple ());
+      noutput_items = round_down (noutput_items, m->output_multiple ());
+      LOG(*d_log << "  max_items_avail = " << max_items_avail << std::endl);
+      LOG(*d_log << "  noutput_items = " << noutput_items << std::endl);
 
-      if (noutput_items == 0)	// we're blocked on input
+      if (noutput_items == 0){	// we're blocked on input
+	LOG(*d_log << "  BLKD_IN\n");
 	goto next_block;
+      }
 
       goto try_again;		// Jump to code shared with regular case.
     }
@@ -185,23 +235,34 @@ gr_single_threaded_scheduler::main_loop ()
 
       // determine the minimum available output space
       noutput_items = min_available_space (d, m->output_multiple ());
+      if (ENABLE_LOGGING){
+	*d_log << " regular ";
+	if (m->relative_rate() >= 1.0)
+	  *d_log << "1:" << m->relative_rate() << std::endl;
+	else
+	  *d_log << 1.0/m->relative_rate() << ":1\n";
+	*d_log << "  max_items_avail = " << max_items_avail << std::endl;
+	*d_log << "  noutput_items = " << noutput_items << std::endl;
+      }
       if (noutput_items == -1)		// we're done
 	goto were_done;
 
-      if (noutput_items == 0)		// we're output blocked
+      if (noutput_items == 0){		// we're output blocked
+	LOG(*d_log << "  BLKD_OUT\n");
 	goto next_block;
-
-      // This isn't necessary for correctness, but saves us time below.
-      // FIXME benchmark to confirm this is worth the trouble
-
-      if (m->relative_rate() < (1.0/8)){	// substantial decimator
-	noutput_items =
-	  std::min ((unsigned) noutput_items,
-		    std::max ((unsigned) m->output_multiple(),
-			      round_up ((unsigned) (max_items_avail * m->relative_rate()),
-					m->output_multiple ())));
       }
 
+#if 0
+      // Compute best estimate of noutput_items that we can really use.
+      noutput_items =
+	std::min ((unsigned) noutput_items,
+		  std::max ((unsigned) m->output_multiple(),
+			    round_up ((unsigned) (max_items_avail * m->relative_rate()),
+				      m->output_multiple ())));
+
+      LOG(*d_log << "  revised noutput_items = " << noutput_items << std::endl);
+#endif
+
     try_again:
       // ask the block how much input they need to produce noutput_items
       m->forecast (noutput_items, ninput_items_required);
@@ -222,6 +283,7 @@ gr_single_threaded_scheduler::main_loop ()
 	}
 
 	// We're blocked on input
+	LOG(*d_log << "  BLKD_IN\n");
 	if (d->input(i)->done())    // If the upstream block is done, we're done
 	  goto were_done;
 
@@ -257,21 +319,33 @@ gr_single_threaded_scheduler::main_loop ()
       // Do the actual work of the block
       int n = m->general_work (noutput_items, ninput_items,
 			       input_items, output_items);
+      LOG(*d_log << "  general_work: noutput_items = " << noutput_items
+	  << " result = " << n << std::endl);
 
       if (n == -1)		// block is done
 	goto were_done;
 
       d->produce_each (n);	// advance write pointers
+      if (n > 0)
+	making_progress = true;
+
       goto next_block;
     }
     assert (0);
     
   were_done:
+    LOG(*d_log << "  were_done\n");
     d->set_done (true);
     nalive--;
 
   next_block:
-    if (++bi >= d_blocks.size ())
+    if (++bi >= d_blocks.size ()){
       bi = 0;
+      made_progress_last_pass = making_progress;
+      making_progress = false;
+    }
   }
+
+  for (unsigned i = 0; i < d_blocks.size (); i++)	// disable any drivers, etc.
+    d_blocks[i]->stop();
 }
