@@ -35,4 +35,10 @@ typedef std::vector<double>			gr_vector_double;
 typedef std::vector<void *>			gr_vector_void_star;
 typedef std::vector<const void *>		gr_vector_const_void_star;
 
+typedef short 		gr_int16;
+typedef int   		gr_int32;
+typedef unsigned short 	gr_uint16;
+typedef unsigned int   	gr_uint32;
+
+
 #endif /* INCLUDED_GR_TYPES_H */
