@@ -57,10 +57,24 @@ def check_intx (option, opt, value):
         raise OptionValueError (
             "option %s: invalid integer value: %r" % (opt, value))
 
+def check_subdev (option, opt, value):
+    """
+    Value has the form: (A|B)(:0|1)?
+
+    @returns a 2-tuple (0|1, 0|1)
+    """
+    d = { 'A'   : (0, 0),  'A:0' : (0, 0),  'A:1' : (0, 1),
+          'B'   : (1, 0),  'B:0' : (1, 0),  'B:1' : (1, 1) }
+    try:
+        return d[value.upper()]
+    except:
+        raise OptionValueError(
+            "option %s: invalid subdev: '%r', must be one of A, B, A:0, A:1, B:0, B:1" % (opt, value))
 
 class eng_option (Option):
-    TYPES = Option.TYPES + ("eng_float", "intx")
+    TYPES = Option.TYPES + ("eng_float", "intx", "subdev")
     TYPE_CHECKER = copy (Option.TYPE_CHECKER)
     TYPE_CHECKER["eng_float"] = check_eng_float
     TYPE_CHECKER["intx"] = check_intx
+    TYPE_CHECKER["subdev"] = check_subdev
 
