@@ -29,4 +29,7 @@ from basic_flow_graph import *
 from flow_graph import *
 from exceptions import *
 from hier_block import *
-# from gnuradio.blks import *
+
+# create a couple of aliases
+stream_to_vector = serial_to_parallel
+vector_to_stream = parallel_to_serial
